@@ -25,16 +25,17 @@
 #include <QAbstractScrollArea>
 #include <QApplication>
 #include <QBoxLayout>
-#include <QDesktopWidget>
 #include <QHeaderView>
 #include <QLabel>
 #include <QPushButton>
-#include <QScopedPointer>
+#include <QScreen>
 #include <QScrollBar>
 #include <QSizeGrip>
 #include <QTimer>
 #include <QToolButton>
 
+#include <KWindowSystem>
+
 const bool hideAutomaticCompletionOnExactMatch = true;
 
 // If this is true, the completion-list is navigated up/down when 'tab' is pressed, instead of doing partial completion
@@ -195,6 +196,7 @@ KateCompletionWidget::~KateCompletionWidget()
 {
     // ensure no slot triggered during destruction => else we access already invalidated stuff
     m_presentationModel->disconnect(this);
+    m_argumentHintModel->disconnect(this);
 }
 
 void KateCompletionWidget::viewFocusOut()
@@ -543,20 +545,20 @@ void KateCompletionWidget::updateAndShow()
     }
 }
 
-bool KateCompletionWidget::updatePosition(bool force)
+void KateCompletionWidget::updatePosition(bool force)
 {
     if (!force && !isCompletionActive()) {
-        return false;
+        return;
     }
 
     if (!completionRange()) {
-        return false;
+        return;
     }
     QPoint cursorPosition = view()->cursorToCoordinate(completionRange()->start());
     if (cursorPosition == QPoint(-1, -1)) {
         // Start of completion range is now off-screen -> abort
         abortCompletion();
-        return false;
+        return;
     }
 
     QPoint p = view()->mapToGlobal(cursorPosition);
@@ -565,23 +567,21 @@ bool KateCompletionWidget::updatePosition(bool force)
 
     y += view()->renderer()->currentFontMetrics().height() + 2;
 
-    bool borderHit = false;
+    if (!KWindowSystem::isPlatformWayland()) {
+        const auto screenGeometry = view()->screen()->availableGeometry();
 
-    if (x + width() > QApplication::desktop()->screenGeometry(view()).right()) {
-        x = QApplication::desktop()->screenGeometry(view()).right() - width();
-        borderHit = true;
-    }
+        if (x + width() > screenGeometry.right()) {
+            x = screenGeometry.right() - width();
+        }
 
-    if (x < QApplication::desktop()->screenGeometry(view()).left()) {
-        x = QApplication::desktop()->screenGeometry(view()).left();
-        borderHit = true;
+        if (x < screenGeometry.left()) {
+            x = screenGeometry.left();
+        }
     }
 
     move(QPoint(x, y));
 
-    //   //qCDebug(LOG_KTE) << "updated to" << geometry() << m_entryList->geometry() << borderHit;
-
-    return borderHit;
+    //   //qCDebug(LOG_KTE) << "updated to" << geometry() << m_entryList->geometry();
 }
 
 void KateCompletionWidget::updateArgumentHintGeometry()
@@ -693,7 +693,7 @@ void KateCompletionWidget::updateHeight()
         baseHeight = m_expandedAddedHeightBase;
     }
 
-    int screenBottom = QApplication::desktop()->screenGeometry(view()).bottom();
+    int screenBottom = view()->screen()->availableGeometry().bottom();
 
     // Limit the height to the bottom of the screen
     int bottomPosition = baseHeight + newExpandingAddedHeight + geometry().top();
@@ -921,7 +921,7 @@ bool KateCompletionWidget::execute()
     m_completionEditRunning = true;
 
     // create scoped pointer, to ensure deletion of cursor
-    QScopedPointer<KTextEditor::MovingCursor> oldPos(view()->doc()->newMovingCursor(view()->cursorPosition(), KTextEditor::MovingCursor::StayOnInsert));
+    std::unique_ptr<KTextEditor::MovingCursor> oldPos(view()->doc()->newMovingCursor(view()->cursorPosition(), KTextEditor::MovingCursor::StayOnInsert));
 
     KTextEditor::CodeCompletionModel *model = static_cast<KTextEditor::CodeCompletionModel *>(const_cast<QAbstractItemModel *>(toExecute.model()));
     Q_ASSERT(model);
@@ -935,6 +935,22 @@ bool KateCompletionWidget::execute()
     std::unique_ptr<KTextEditor::MovingCursor> afterTailMCursor(view()->doc()->newMovingCursor(view()->cursorPosition()));
     afterTailMCursor->move(tailStr.size());
 
+    // Handle completion for multi cursors
+    QSharedPointer<QMetaObject::Connection> connection(new QMetaObject::Connection());
+    auto autoCompleteMulticursors = [connection, this](KTextEditor::Document *document, const KTextEditor::Range &range) {
+        disconnect(*connection);
+        const QString text = document->text(range);
+        if (text.isEmpty()) {
+            return;
+        }
+        const auto &multicursors = view()->secondaryCursors();
+        for (const auto &c : multicursors) {
+            KTextEditor::Range wordToReplace = view()->doc()->wordRangeAt(c.cursor());
+            view()->doc()->replaceText(wordToReplace, text);
+        }
+    };
+    *connection = connect(view()->doc(), &KTextEditor::DocumentPrivate::textInsertedRange, this, autoCompleteMulticursors);
+
     model->executeCompletionItem(view(), *m_completionRanges[model].range, toExecute);
     // NOTE the CompletionRange is now removed from m_completionRanges
 
@@ -1362,6 +1378,21 @@ int KateCompletionWidget::automaticInvocationDelay() const
     return m_automaticInvocationDelay;
 }
 
+void KateCompletionWidget::setIgnoreBufferSignals(bool ignore)
+{
+    if (ignore) {
+        disconnect(&view()->doc()->buffer(), &KateBuffer::lineWrapped, this, &KateCompletionWidget::wrapLine);
+        disconnect(&view()->doc()->buffer(), &KateBuffer::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
+        disconnect(&view()->doc()->buffer(), &KateBuffer::textInserted, this, &KateCompletionWidget::insertText);
+        disconnect(&view()->doc()->buffer(), &KateBuffer::textRemoved, this, &KateCompletionWidget::removeText);
+    } else {
+        connect(&view()->doc()->buffer(), &KateBuffer::lineWrapped, this, &KateCompletionWidget::wrapLine);
+        connect(&view()->doc()->buffer(), &KateBuffer::lineUnwrapped, this, &KateCompletionWidget::unwrapLine);
+        connect(&view()->doc()->buffer(), &KateBuffer::textInserted, this, &KateCompletionWidget::insertText);
+        connect(&view()->doc()->buffer(), &KateBuffer::textRemoved, this, &KateCompletionWidget::removeText);
+    }
+}
+
 void KateCompletionWidget::setAutomaticInvocationDelay(int delay)
 {
     m_automaticInvocationDelay = delay;
